/*******************************************************************************
 *
 * amrecGUI Version 0.2.2
 *
 * amrecgui is a simple frontend for amrecover, the command-line recovery tool for the backup suite amanda. amrecgui is licensed under GPL conditions.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "amrecguimainframe.h"
#include "recoverview.h"
#include <basic/globalstring.h>
#include <gui/dataspoolerrootdialog.h>
#include <basic/date.h>
#include "datasetmanager.h"

CAmrecguiMainframe::CAmrecguiMainframe()
{
    m_pRecoverView = NULL;

    m_Configuration.SetSpooler(&m_Dataspooler);
}

const char *CAmrecguiMainframe::PropertiesName() const
{
#ifdef _DEBUG
    return ".amrecguidbgrc";
#else
    return ".amrecguirc";
#endif
}

const char *CAmrecguiMainframe::Title() const
{
    return "amrecGUI";
}

const char *CAmrecguiMainframe::Version() const
{
    return "0.2.3";
}

void CAmrecguiMainframe::InternalCreate(unsigned long lStyle) 
{
    m_pRecoverView = new CRecoverView(&m_Configuration);
    m_pRecoverView->SetCaption(LANGUAGE_STRING(IDS_RECOVER));
    *this += m_pRecoverView;

    CMainFrame::InternalCreate(lStyle);

    CStrings strRootDirectory = Properties()->GetEntry("RootDirectory");

    if (strRootDirectory == "")
    {
	CDataSpoolerRootDialog dialog;

	dialog.Create();

	int nResult = dialog.DoModal();

	if (nResult == CDialog::IDOK)
	{
	    strRootDirectory = dialog.Path();

	    if (strRootDirectory != "")
	    {
		Properties()->SetEntry("RootDirectory", strRootDirectory);
	    }
	    else
	    {
		exit(1);
	    }
	}
	else
	{
	    exit(1);
	}
    }

    m_Dataspooler.SetRoot(strRootDirectory);

    CStrings strDataSet = Properties()->GetEntry("CurrentDataSet");
    int nDataSet = 0;
    
    if (strDataSet == "") 
    {
	CDatasetManager manager(&m_Dataspooler);
	manager.Create();

	int nResult = manager.DoModal();

	if (nResult == CDialog::IDOK)
	{
	    nDataSet = manager.GetSelection();
	}
	else
	{
	    exit(1);
	}
    }
    else
    {
	nDataSet = atoi(strDataSet);
    }

    CStrings strDisk = Properties()->GetEntry("CurrentDisk");
    int nDisk = 0;

    if (strDisk == "")
    {
	nDisk = 1;
    }
    else
    {
	nDisk = atoi(strDisk);
    }

    CStrings strOutput = Properties()->GetEntry("Output");
    
    if (strOutput == "")
    {
	CFileSelectionDialog dialog;

	dialog.Create();
	dialog.SetCaption(LANGUAGE_STRING(IDS_SETWORKINGDIRECTORY));

	int nResult = dialog.DoModal();

	if (nResult == CDialog::IDOK)
	{
	    strOutput = dialog.Path();
	}
	else
	{
	    exit(1);
	}
    }
    
    m_Configuration.Load(nDataSet, CDate::Today(), nDisk, strOutput);
}

void CAmrecguiMainframe::InternalDestroy()
{
    CDate dtDate = m_Configuration.Date();
    IDENT dataset = m_Configuration.DataSet();
    IDENT disk = m_Configuration.Disk();
    CStrings strOutput = m_Configuration.Output();

    Properties()->SetEntry("CurrentDataSet", (int)dataset);
    Properties()->SetEntry("CurrentDisk", (int)disk);
    Properties()->SetEntry("Output", strOutput);

    CMainFrame::InternalDestroy();
}




