/*******************************************************************************
 *
 * amrecGUI Version 0.2.2
 *
 * amrecgui is a simple frontend for amrecover, the command-line recovery tool for the backup suite amanda. amrecgui is licensed under GPL conditions.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "datasetmanager.h"
#include <basic/dataspooler.h>
#include <gui/selectionlist.h>
#include <gui/button.h>
#include <basic/globalstring.h>
#include "editdatasetdialog.h"
#include <basic/table.h>
#include "datasets.h"

CDatasetManager::CDatasetManager(CDataSpooler *pDataSpooler)
    : CModalDialog()
{
    m_pDataSpooler = pDataSpooler;
    m_pSelectionList = NULL;
    m_pNewButton = NULL;
    m_pEditButton = NULL;
    m_pEraseButton = NULL;
    m_nSelection = 0;
}

IDENT CDatasetManager::GetSelection() 
{
    return m_nSelection;
}

void CDatasetManager::InternalCreate(unsigned long lStyle)
{
    CModalDialog::InternalCreate(lStyle);

    Arg args[20];
    Cardinal argcount = 0;

    m_pSelectionList = new CSelectionList(Frame());

    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNbottomAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNbottomWidget, Separator()); argcount++;
    XtSetArg(args[argcount], XmNwidth, 300); argcount++;

    m_pSelectionList->Create(0, args, argcount);

    argcount = 0;

    m_pNewButton = new CButton(Frame(), ID_NEW);
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNleftWidget, m_pSelectionList->Container()); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
    m_pNewButton->Create(0L, args, argcount);
    m_pNewButton->SetCaption(LANGUAGE_STRING(IDS_NEW));

    m_pEditButton = new CButton(Frame(), ID_EDIT);
    argcount = 0;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNleftWidget, m_pSelectionList->Container()); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNtopWidget, m_pNewButton->Container()); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
    m_pEditButton->Create(0L, args, argcount);
    m_pEditButton->SetCaption(LANGUAGE_STRING(IDS_EDIT));

    m_pEraseButton = new CButton(Frame(), ID_ERASE);
    argcount = 0;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNleftWidget, m_pSelectionList->Container()); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNtopWidget, m_pEditButton->Container()); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
    m_pEraseButton->Create(0L, args, argcount);
    m_pEraseButton->SetCaption(LANGUAGE_STRING(IDS_DELETE));

    ListenTo(m_pNewButton);
    ListenTo(m_pEditButton);
    ListenTo(m_pEraseButton);

    ReadOut();

    SetCaption(LANGUAGE_STRING(IDS_DATASET_WAEHLEN));
}
void CDatasetManager::OnMessage(CMessageClient *pClient, unsigned long lMask, void *pData)
{
    switch(pClient->Id())
    {
	case ID_EDIT:
	{
	    CIdString *pString = m_pSelectionList->GetCurrentSelection();
	    if (pString != NULL)
	    {
		CTable Table;

		if (m_pDataSpooler->OpenTable(Table, "Datasets"))
		{		
		    if (Table.Seek(pString->Id()) == CTable::TS_OK)
		    {
			CStrings strHostname;
			CStrings strDataset;
			
			Table.Get(ID_DATASETS_HOSTNAME, strHostname);
			Table.Get(ID_DATASETS_DATASET, strDataset);

			CEditDataSetDialog dialog(m_pDataSpooler, pString->Id());
			dialog.Create();

			int nResult = dialog.DoModal();

			if (nResult == CDialog::IDOK)
			{
			    m_pSelectionList->SetCurrentSelection(dialog.Name());

			    Table.Set(ID_DATASETS_NAME, *pString);
			    Table.Set(ID_DATASETS_HOSTNAME, dialog.Hostname());
			    Table.Set(ID_DATASETS_DATASET, dialog.Dataset());
			}
		    }
		}
	    }
	}
	break;
	case ID_NEW:
	{
	    CTable Table;
	    if (!m_pDataSpooler->OpenTable(Table, "Datasets"))
	    {
		if (m_pDataSpooler->CreateTable(ID_DATASETS, "Datasets"))
		{
		    if (!m_pDataSpooler->OpenTable(Table, "Datasets"))
		    {
			exit(2);
		    }
		}
	    }

	    int nId = Table.NextId();
	    
	    Table.AppendRecord(nId);
	    
	    CEditDataSetDialog dialog(m_pDataSpooler, nId);
	    
	    dialog.Create();
	    
	    int nResult = dialog.DoModal();
	    
	    if (nResult == CDialog::IDOK)
	    {		
		if (Table.Seek(nId) == CTable::TS_OK)
		{
		    Table.Set(ID_DATASETS_NAME, dialog.Name());
		    Table.Set(ID_DATASETS_HOSTNAME, dialog.Hostname());
		    Table.Set(ID_DATASETS_DATASET, dialog.Dataset());
		    
		    CIdString string;
		    string = dialog.Name();
		    string.SetId(nId);
		    
		    *m_pSelectionList += string;
		}
	    }
	}
	break;
	case ID_ERASE:
	{
	    CIdString *pString = m_pSelectionList->GetCurrentSelection();

	    if (pString != NULL)
	    {
		CTable Table;
		if (m_pDataSpooler->OpenTable(Table, "Datasets"))
		{
		    if (Table.Seek(pString->Id()) == CTable::TS_OK)
		    {
			Table.DeleteRecord();
			*m_pSelectionList -= *pString;
		    }
		}
	    }
	}
	break;
    }
}	

void CDatasetManager::ReadOut()
{
    CTable Table;

    if (m_pDataSpooler->OpenTable(Table, "Datasets"))
    {
	for (Table.Top(); !Table.Eof(); Table.Skip())
	{
	    CIdString string;

	    Table.Get(ID_DATASETS_NAME, string);
	    string.SetId(Table.CurrentId());
	    
	    *m_pSelectionList += string;
	}
    }
}

void CDatasetManager::InternalDestroy()
{
    delete m_pNewButton;
    delete m_pEditButton;
    delete m_pEraseButton;
    delete m_pSelectionList;

    CModalDialog::InternalDestroy();
}

void CDatasetManager::OnOk()
{
    if (m_pSelectionList->GetCurrentSelection() != NULL)
    {
	m_nSelection = m_pSelectionList->GetCurrentSelection()->Id();
	
	CModalDialog::OnOk();
    }
}
