/*******************************************************************************
 *
 * amrecGUI Version 0.2.2
 *
 * amrecgui is a simple frontend for amrecover, the command-line recovery tool for the backup suite amanda. amrecgui is licensed under GPL conditions.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "editdatasetdialog.h"
#include <gui/labelledtextfield.h>
#include <basic/globalstring.h>
#include <gui/selectionlist.h>
#include <gui/button.h>
#include <basic/dataspooler.h>
#include <basic/macrodefinitions.h>
#include <basic/table.h>
#include "editdiskdialog.h"
#include "datasets.h"

enum {ID_NEW, ID_EDIT, ID_DELETE};

CEditDataSetDialog::CEditDataSetDialog(CDataSpooler *pDataSpooler, int nId)
    : CModalDialog()
{
    m_nId = nId;
    m_pDataSpooler = pDataSpooler;
}

void CEditDataSetDialog::InternalCreate(unsigned long lStyle)
{
    CModalDialog::InternalCreate(lStyle);

    m_pName = new CLabelledTextField(Frame());

    Arg args[20];
    Cardinal argcount = 0;

    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_FORM); argcount++;

    m_pName->Create(0L, args, argcount);

    m_pName->SetCaption(LANGUAGE_STRING(IDS_NAME));

    argcount = 0;

    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNtopWidget, m_pName->Container()); argcount++;

    m_pHostname = new CLabelledTextField(Frame());
    m_pHostname->Create(0L, args, argcount);
    m_pHostname->SetCaption(LANGUAGE_STRING(IDS_HOSTNAME));

    argcount = 0;

    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNtopWidget, m_pHostname->Container()); argcount++;

    m_pDataset = new CLabelledTextField(Frame());
    m_pDataset->Create(0L, args, argcount);
    m_pDataset->SetCaption(LANGUAGE_STRING(IDS_DATASET));

    SetCaption(LANGUAGE_STRING(IDS_EDIT_DATASET));

    argcount = 0;

    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNtopWidget, m_pDataset->Container()); argcount++;
    XtSetArg(args[argcount], XmNbottomAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNbottomWidget, Separator()); argcount++;

    m_pSelectionList = new CSelectionList(Frame());
    m_pSelectionList->Create(0L, args, argcount);

    argcount = 0;

    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNleftWidget, m_pSelectionList->Container()); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNtopWidget, m_pDataset->Container()); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;

    m_pNew = new CButton(Frame(), ID_NEW);
    m_pNew->Create(0L, args, argcount);
    m_pNew->SetCaption(LANGUAGE_STRING(IDS_NEW));

    argcount = 0;

    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNleftWidget, m_pSelectionList->Container()); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNtopWidget, m_pNew->Container()); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;

    m_pEdit = new CButton(Frame(), ID_EDIT);
    m_pEdit->Create(0L, args, argcount);
    m_pEdit->SetCaption(LANGUAGE_STRING(IDS_EDIT));

    argcount = 0;

    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNleftWidget, m_pSelectionList->Container()); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNtopWidget, m_pEdit->Container()); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;

    m_pDelete = new CButton(Frame(), ID_DELETE);
    m_pDelete->Create(0L, args, argcount);    
    m_pDelete->SetCaption(LANGUAGE_STRING(IDS_DELETE));

    ListenTo(m_pNew);
    ListenTo(m_pEdit);
    ListenTo(m_pDelete);

    ReadOut();
}

void CEditDataSetDialog::OnOk()
{
    m_strName = m_pName->Value();
    m_strHostname = m_pHostname->Value();
    m_strDataset = m_pDataset->Value();

    if (m_strName != "" && m_strHostname != "" && m_strDataset != "")
    {
	CModalDialog::OnOk();
    }
}

void CEditDataSetDialog::InternalDestroy()
{
    delete m_pName;
    delete m_pHostname;
    delete m_pDataset;
    delete m_pSelectionList;
    delete m_pNew;
    delete m_pEdit;
    delete m_pDelete;
}

void CEditDataSetDialog::ReadOut()
{
    CTable Table;

    if (m_pDataSpooler->OpenTable(Table, "Datasets"))
    {
	if (Table.Seek(m_nId) == CTable::TS_OK)
	{
	    Table.Get(ID_DATASETS_NAME, m_strName);
	    Table.Get(ID_DATASETS_HOSTNAME, m_strHostname);
	    Table.Get(ID_DATASETS_DATASET, m_strDataset);
	}
    }

    m_pName->SetValue(m_strName);
    m_pHostname->SetValue(m_strHostname);
    m_pDataset->SetValue(m_strDataset);

    if (!m_pDataSpooler->OpenTable(Table, "Disks"))
    {    
	if (m_pDataSpooler->CreateTable(ID_DISKS, "Disks"))
	{
	    if (m_pDataSpooler->OpenTable(Table, "Disks"))
	    {
		Table.AppendRecord(m_nId, Table.NextSecondId(m_nId));
	    
		CEditDiskDialog dialog("---DISK---");
		dialog.Create();
	    
		int nResult = dialog.DoModal();

		if (nResult == CDialog::IDOK)
		{
		    Table.Set(ID_DISKS_NAME, dialog.Name());
		}
		else
		{
		    Table.DeleteRecord();
		}
	    }
	}
	else
	{
	    exit(2);
	}
    }

    if (Table.FindFirst(m_nId)) 
    {
	do
	{
	    CIdString string;
	    Table.Get(ID_DISKS_NAME, string);
	    string.SetId(Table.SecondId());
	    
	    *m_pSelectionList += string;
	} while(Table.FindNext(m_nId));
    }
}

void CEditDataSetDialog::OnMessage(CMessageClient *pClient, unsigned long lMask, void *pData)
{
    switch(pClient->Id())
    {
	case ID_NEW:
	{
	    CTable Table;

	    bool bOk = true;

	    if (!m_pDataSpooler->OpenTable(Table, "Disks"))
	    {    
		if (m_pDataSpooler->CreateTable(ID_DISKS, "Disks"))
		{
		    if (!m_pDataSpooler->OpenTable(Table, "Disks"))
		    {
			bOk = false;
		    }
		}
	    }

	    if (bOk)
	    {
		int nId = Table.NextSecondId(m_nId);
		Table.AppendRecord(m_nId, nId);
	    
		CEditDiskDialog dialog("---DISK---");
		dialog.Create();
	    
		int nResult = dialog.DoModal();

		if (nResult == CDialog::IDOK)
		{
		    Table.Set(ID_DISKS_NAME, dialog.Name());
		    CIdString string;
		    string = dialog.Name();
		    string.SetId(nId);
		    *m_pSelectionList += string;
		}
		else
		{
		    Table.DeleteRecord();
		}
	    }
	}
	break;
	case ID_EDIT:
	{
	    CTable Table;

	    bool bOk = true;

	    if (!m_pDataSpooler->OpenTable(Table, "Disks"))
	    {    
		bOk = false;
	    }

	    if (bOk)
	    {
		CIdString *pString = m_pSelectionList->GetCurrentSelection();

		if (pString != NULL)
		{
		    Table.Seek(m_nId, pString->Id());
	    
		    CEditDiskDialog dialog(*pString);
		    dialog.Create();
	    
		    int nResult = dialog.DoModal();

		    if (nResult == CDialog::IDOK)
		    {
			Table.Set(ID_DISKS_NAME, dialog.Name());
			m_pSelectionList->SetCurrentSelection(dialog.Name());
		    }
		}
	    }
	}
	break;
	case ID_DELETE:
	{
	    CTable Table;

	    if (m_pDataSpooler->OpenTable(Table, "Disks"))
	    {    
		CIdString *pString = m_pSelectionList->GetCurrentSelection();

		if (pString != NULL)
		{
		    if (Table.Seek(m_nId, pString->Id()) == CTable::TS_OK)
		    {
			Table.DeleteRecord();
			*m_pSelectionList -= *pString;
		    }
		}
	    }
	}
	break;
    }
}




