/*******************************************************************************
 *
 * amrecGUI Version 0.2.2
 *
 * amrecgui is a simple frontend for amrecover, the command-line recovery tool for the backup suite amanda. amrecgui is licensed under GPL conditions.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "recoverview.h"
#include "datasetconfiguration.h"
#include <Xm/PanedW.h>
#include <Xm/Text.h>
#include <gui/button.h>
#include <basic/globalstring.h>
#include <Xm/Form.h>
#include "datasetmanager.h"
#include "editdatasetdialog.h"
#include <gui/labelleddatetextfield.h>
#include <gui/selectionlist.h>
#include <gui/frame.h>
#include <gui/shellinterface.h>
#include <basic/table.h>
#include <basic/dataspooler.h>
#include <gui/filebutton.h>
#include <unistd.h>
#include <basic/stringtokenizer.h>
#include <gui/labelledtextfield.h>
#include <basic/ctree.h>
#include "recoveraskdialog.h"
#include <gui/messagedialog.h>
#include "recovertreebox.h"
#include <gui/waitcursor.h>
#include "datasets.h"

enum {ID_DATASET, ID_PROPERTIES, ID_COMMIT, ID_RECOVER, ID_CONNECT, ID_DISCONNECT};

CRecoverView::CRecoverView(CDataSetConfiguration *pConfiguration)
{
    m_pConfiguration = pConfiguration;
    m_pFilesBox = NULL;
    SetConnected(false);
}

void CRecoverView::InternalCreate(unsigned long lStyle)
{
    Arg args[20];
    Cardinal argcount = 0;

    XtSetArg(args[argcount], XmNorientation, XmHORIZONTAL); argcount++;
    Widget wdgVerticalWindow = XmCreatePanedWindow(Parent(), "topform", args, argcount);
    XtManageChild(wdgVerticalWindow);

    argcount = 0;
    CTreeNode<CIdString> tree;

    //m_pFilesBox = new CHierarchicalTreeBox(wdgVerticalWindow, HIERARCHICAL_LINES|HIERARCHICAL_CHECK_BOX|HIERARCHICAL_READONLY);
    m_pFilesBox = new CRecoverTreeBox(wdgVerticalWindow, HIERARCHICAL_LINES|HIERARCHICAL_CHECK_BOX|HIERARCHICAL_READONLY);
    m_pFilesBox->Create(&tree);

    argcount = 0;
    Widget wdgRightForm = XmCreateForm(wdgVerticalWindow, "rightform", args, argcount);
    XtManageChild(wdgRightForm);

    argcount = 0;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_FORM); argcount++;
    
    Widget wdgButtonForm = XmCreateForm(wdgRightForm, "buttonform", args, argcount);
    XtManageChild(wdgButtonForm);

    argcount = 0;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNwidth, 120); argcount++;

    m_pDatasetButton = new CButton(wdgButtonForm, ID_DATASET);
    m_pDatasetButton->Create(0L, args, argcount);

    m_pDatasetButton->SetCaption(LANGUAGE_STRING(IDS_DATASET_WAEHLEN));
    ListenTo(m_pDatasetButton);

    argcount = 0;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNtopWidget, m_pDatasetButton->Container()); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNwidth, 120); argcount++;
    XtSetArg(args[argcount], XmNbottomAttachment, XmATTACH_FORM); argcount++;
   
    m_pProperties = new CButton(wdgButtonForm, ID_PROPERTIES);
    m_pProperties->Create(0L, args, argcount);
    m_pProperties->SetCaption(LANGUAGE_STRING(IDS_PROPERTIES));
    ListenTo(m_pProperties);

    argcount = 0;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_NONE); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_FORM); argcount++;
    
//      Widget wdgDataForm = XmCreateForm(wdgRightForm, "dataform", args, argcount);
//      XtManageChild(wdgDataForm);
    CFrame *pFrame = new CFrame(wdgRightForm);
    pFrame->Create(0L, args, argcount);
    pFrame->SetCaption(LANGUAGE_STRING(IDS_SETTINGS));

    argcount = 0;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_FORM); argcount++;
    //XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;

    m_pDate = new CLabelledDateTextField(pFrame->Frame());
    m_pDate->Create(0L, args, argcount);

    m_pDate->SetCaption(LANGUAGE_STRING(IDS_DATE));

    argcount = 0;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNtopWidget, m_pDate->Container()); argcount++;
   
    m_pOutput = new CFileButton(pFrame->Frame());
    m_pOutput->Create(0L, args, argcount);

    argcount = 0;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNtopWidget, m_pOutput->Container()); argcount++;
    
    m_pPath = new CLabelledTextField(pFrame->Frame());
    m_pPath->Create(0L, args, argcount);
    m_pPath->SetCaption(LANGUAGE_STRING(IDS_PATH));

    argcount = 0;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNtopWidget, m_pPath->Container()); argcount++;

    m_pSelectionList = new CSelectionList(pFrame->Frame());
    m_pSelectionList->Create(0L, args, argcount);

    argcount = 0;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNtopWidget, m_pSelectionList->Container()); argcount++;
    XtSetArg(args[argcount], XmNbottomAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNwidth, 300); argcount++;
    
    m_pCommit = new CButton(pFrame->Frame(), ID_COMMIT);
    m_pCommit->Create(0L, args, argcount);
    m_pCommit->SetCaption(LANGUAGE_STRING(IDS_COMMIT));

    ListenTo(m_pCommit);

    argcount = 0;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNrightAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNbottomAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNtopWidget, pFrame->Container()); argcount++;

    m_pInterface = new CShellInterface(wdgRightForm);
    m_pInterface->Create(0L, args, argcount);

    argcount = 0;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_NONE); argcount++;
    XtSetArg(args[argcount], XmNbottomAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNbottomWidget, m_pInterface->Container()); argcount++;
    XtSetArg(args[argcount], XmNwidth, 120); argcount++;

    m_pRecover = new CButton(wdgRightForm, ID_RECOVER);
    m_pRecover->Create(0L, args, argcount);
    m_pRecover->SetCaption(LANGUAGE_STRING(IDS_RECOVER));
    ListenTo(m_pRecover);

    argcount = 0;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_NONE); argcount++;
    XtSetArg(args[argcount], XmNbottomAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNbottomWidget, m_pRecover->Container()); argcount++;
    XtSetArg(args[argcount], XmNwidth, 120); argcount++;

    m_pConnect = new CButton(wdgRightForm, ID_CONNECT);
    m_pConnect->Create(0L, args, argcount);
    m_pConnect->SetCaption(LANGUAGE_STRING(IDS_CONNECT));
    ListenTo(m_pConnect);

    argcount = 0;
    XtSetArg(args[argcount], XmNleftAttachment, XmATTACH_FORM); argcount++;
    XtSetArg(args[argcount], XmNtopAttachment, XmATTACH_NONE); argcount++;
    XtSetArg(args[argcount], XmNbottomAttachment, XmATTACH_WIDGET); argcount++;
    XtSetArg(args[argcount], XmNbottomWidget, m_pConnect->Container()); argcount++;
    XtSetArg(args[argcount], XmNwidth, 120); argcount++;

    m_pDisconnect = new CButton(wdgRightForm, ID_DISCONNECT);
    m_pDisconnect->Create(0L, args, argcount);
    m_pDisconnect->SetCaption(LANGUAGE_STRING(IDS_DISCONNECT));
    ListenTo(m_pDisconnect);

    Init();

    m_pDisconnect->Enable(false);
    m_pRecover->Enable(false);
    m_pCommit->Enable(false);

    ListenTo(m_pFilesBox);
}

void CRecoverView::OnMessage(CMessageClient *pClient, unsigned long lMask, void *pData)
{
    if (lMask & EVENT_MASK_BUTTON_PUSHED)
    {
	switch(pClient->Id())
	{
	    case ID_DATASET:
	    {
		CDatasetManager manager(m_pConfiguration->DataSpooler());
		manager.Create();

		int nResult = manager.DoModal();

		if (nResult == CDialog::IDOK)
		{
		    int nId = manager.GetSelection();

		    m_pConfiguration->Load(nId, CDate::Today(), 1, m_pConfiguration->Output());
		    Init();
		}
	    }
	    break;
	    case ID_PROPERTIES:
	    {
		CEditDataSetDialog dialog(m_pConfiguration->DataSpooler(), m_pConfiguration->DataSet());
		dialog.Create();
	    
		int nResult = dialog.DoModal();

		if (nResult == CDialog::IDOK)
		{
		    Init();
		}
	    }
	    break;
	    case ID_COMMIT:
	    {
		CIdString *pString = m_pSelectionList->GetCurrentSelection();

		if (pString != NULL)
		{
		    CDate date = m_pDate->Value();
		
		    IDENT nId = pString->Id();

		    m_pConfiguration->SetDate(date);
		    m_pConfiguration->SetDisk(pString->Id());
		    m_pConfiguration->SetOutput(m_pOutput->Path());

		    Init();

		    m_pSelectionList->Select(nId);

		    if (Connected())
		    {
			Commit();
		    }
		}
	    }
	    break;
	    case ID_RECOVER:
	    {
		Recover();
	    }
	    break;
	    case ID_CONNECT:
		if (!Connected())
		{
		    Connect();
		    Commit();
		}
		break;
	    case ID_DISCONNECT:
		Disconnect();
		break;
	}
    }
    else if (lMask & EVENT_MASK_VALUE_UPDATE)
    {
	if (pClient == m_pFilesBox)
	{
//  	    if (pData != NULL)
//  	    {
	    CGraphicalTreeNode *pNode = (CGraphicalTreeNode *)pData;
	    
//  		if (!m_LoadedNodes.in(pNode))
//  		{
	    UpdateListing(pNode);

//  		    m_LoadedNodes += pNode;
//  		}
//  	    }
	}
    }
}

void CRecoverView::Init()
{
    CWaitCursor cursor(Parent());

    m_pSelectionList->ResetContents();
    m_LoadedNodes.reset();

    CTable Table;

    if (m_pConfiguration->DataSpooler()->OpenTable(Table, "Disks"))
    {
	if (Table.FindFirst(m_pConfiguration->DataSet()))
	{
	    do
	    {
		CIdString string;
		Table.Get(ID_DISKS_NAME, string);
		string.SetId(Table.SecondId());

		*m_pSelectionList += string;
	    } while(Table.FindNext(m_pConfiguration->DataSet()));
	}
    }

    m_pDate->SetValue(m_pConfiguration->Date());
    m_pOutput->SetPath(m_pConfiguration->Output());
    m_pSelectionList->Select((IDENT)m_pConfiguration->Disk());
}

void CRecoverView::InternalDestroy()
{
    if (Connected())
    {
  	Disconnect();
    }
    delete m_pFilesBox;
    delete m_pDatasetButton;
    delete m_pProperties;
    delete m_pDate;
    delete m_pSelectionList;
    delete m_pCommit;
    delete m_pOutput;
    delete m_pRecover;
    delete m_pPath;
}

void CRecoverView::Commit()
{
    if (Connected())
    {
	CWaitCursor cursor(Parent());

	CStrings strDate;
	strDate.Format("setdate %04d-%02d-%02d\n", m_pConfiguration->Date().Year(), m_pConfiguration->Date().Month(), m_pConfiguration->Date().Day());
	Write(strDate);
	CIdString *pString = m_pSelectionList->GetCurrentSelection();
	m_strDisk = *pString;
	CStrings strDisk = "setdisk "+*pString+"\n";
	Write(strDisk);
	CStrings strDirectory = "lcd \""+m_pConfiguration->Output()+"\"\n";
	Write(strDirectory);
	CStrings strJumpTo = m_pPath->Value();

	if (strJumpTo != "")
	{
	    CStringTokenizer tokenizer("/");
	    tokenizer << strJumpTo;
	    CStrings strPart;
	    while (tokenizer.NextToken(strPart))
	    {
		CStrings strCommand = "cd \""+strPart+"\"\n";
		Write(strCommand);
	    }

	    m_strPath = strJumpTo;
	    if (m_strPath.Right(1) != "/")
	    {
		m_strPath += "/";
	    }
	}
	ReadTree();
    }
}

void CRecoverView::Connect()
{
    if (!Connected())
    {
	CWaitCursor cursor(Parent());

	SetConnected(true);

	pipe(m_fds);
	pipe(m_outfds);

	pid_t pid = fork();

	if (pid == (pid_t)0) 
	{
	    close(m_fds[1]);
	    close(m_outfds[0]);

	    setreuid(0,0);
	    
	    dup2(m_fds[0], STDIN_FILENO);
	    dup2(m_outfds[1], STDOUT_FILENO);
	    dup2(m_outfds[1], STDERR_FILENO);

	    int nId = m_pConfiguration->DataSet();
	    
	    if (nId > 0)
	    {
		CTable Table;
	    
		if (m_pConfiguration->DataSpooler()->OpenTable(Table, "Datasets"))
		{
		    if (Table.Seek(nId) == CTable::TS_OK)
		    {
			CStrings strDataset;
			CStrings strHostname;

			Table.Get(ID_DATASETS_DATASET, strDataset);
			Table.Get(ID_DATASETS_HOSTNAME, strHostname);

			execlp("/usr/sbin/amrecover", "amrecover", (const char *)strDataset, "-s", (const char *)strHostname, "-t", (const char *)strHostname, NULL);
		    }
		}
	    }
	    
	    exit(0);
	}

	close(m_fds[0]);
	close(m_outfds[1]);

	m_Command = fdopen(m_fds[1], "w");
	m_Output = fdopen(m_outfds[0], "r");

	ASSERT(m_Command != NULL);
	ASSERT(m_Output != NULL);

	Read();

	m_pCommit->Enable();
	m_pConnect->Enable(false);
	m_pDisconnect->Enable();
	m_pRecover->Enable();
	m_pDatasetButton->Enable(false);
	m_pProperties->Enable(false);
    }
}

CStrings CRecoverView::Read() return strRead
{
    CStrings strLine;

    int nChar;

    do
    {
	nChar = fgetc(m_Output);
//  #ifdef _DEBUG
//  	putc(nChar, stderr);
//  #endif

	if (nChar != EOF)
	{
	    strLine += nChar;
	}

	if (nChar == '\n')
	{
	    *m_pInterface += (char *)(const char *)strLine;
	    strRead += strLine+"\n";
	    strLine = "";
	}
	else if (strLine.Right(10) == "amrecover>")
	{
	    *m_pInterface += (char *)(const char *)strLine; 
	    nChar = EOF;
	    strRead += strLine;
	}
	else if (strLine.Right(16) == "Continue? [Y/n]:")
	{
	    *m_pInterface += (char *)(const char *)strLine; 
	    nChar = EOF;
	    strRead += strLine;
	}
	else if (strLine.Right(28) == "set owner/mode for \'.\'? [yn]")
	{
	    *m_pInterface += (char *)(const char *)strLine; 
	    nChar = EOF;
	    strRead += strLine;
	}
    } while (nChar != EOF);
}

CStrings CRecoverView::Write(const char *strCommand, bool bRead) return strRead
{
    CStrings strLine = strCommand;

    //*m_pInterface += (char *)(const char *)(strLine+"\n");
    fputs(strLine, m_Command);
    fflush(m_Command);
    if (bRead)
    {
	strRead = Read();
    }
}

void CRecoverView::Disconnect()
{
    if(Connected())
    {
	CWaitCursor cursor(Parent());

	if (m_Command != NULL)
	{
//  	    fputs("quit\n", m_Command); 
//  	    fflush(m_Command);
	    Write("quit", false);
	    *m_pInterface += "quit\nClient disconnected.\n***\n";

	    fclose(m_Command);
	    m_Command = NULL;
	    fclose(m_Output);
	    m_Output = NULL;
	}
  	close(m_fds[1]);
  	close(m_outfds[0]);

	SetConnected(false);

	m_pFilesBox->DeleteAllItems();

	m_pCommit->Enable(false);
	m_pConnect->Enable();
	m_pDisconnect->Enable(false);
	m_pRecover->Enable(false);
	m_pDatasetButton->Enable();
	m_pProperties->Enable();
    }
}

void CRecoverView::ReadTree()
{
    m_pFilesBox->DeleteAllItems();
    m_nId = 1;

    CTreeNode<CIdString> *pTree = new CTreeNode<CIdString>;

    CStrings strDirectory = Write("ls\n");
	
    WorkListing(pTree, strDirectory, 2);

    m_pFilesBox->Create(pTree);

    delete pTree;
}

void CRecoverView::UpdateListing(CGraphicalTreeNode *pNode)
{
    if (pNode != NULL)
    {
	CWaitCursor cursor(Parent());

	int nIndex = 0;
	CGraphicalTreeNode *pChild = NULL;

	if (!m_LoadedNodes.in(pNode))
	{
	    bool bDir = false;

	    while ((pChild = pNode->Child(nIndex++)))
	    {
		UpdateChildListing(pChild, bDir);

		if (pChild->Node().Right(1) == "/")
		{
		    bDir = true;
		    Sync();
		}
	    }

	    m_LoadedNodes += pNode;
	}
    }
}

void CRecoverView::UpdateChildListing(CGraphicalTreeNode *pNode, bool bReuseDir)
{
    if (pNode->Node().Right(1) == "/")
    {
	if (!bReuseDir)
	{
	    Write("cd \""+m_strDisk+"\"\n");

	    CStrings strDirectory = FilePath(pNode, false);

	    CStringTokenizer tokenizer("/");
	    tokenizer << strDirectory;

	    CStrings strToken;
	    while (tokenizer.NextToken(strToken))
	    {
		if (strToken != "")
		{
		    Write("cd \""+strToken+"\"\n");
		}
	    }
	}
	else
	{
	    Write("cd ..\n");
	}

	Write("cd \""+File(pNode)+"\"\n");

	CStrings strListing = Write("ls\n");

	CTreeNode<CIdString> TopNode;

	WorkListing(&TopNode, strListing, 1);

	m_pFilesBox->Insert(pNode, &TopNode);

    }
}

void CRecoverView::WorkListing(CTreeNode<CIdString> *pTree, const CStrings &strDirectory, int nPrune)
{
    if (nPrune != 0)
    {
	nPrune--;

	CStringTokenizer tokenizer("\n");

	tokenizer << strDirectory;

	CStrings strToken;
	tokenizer.NextToken(strToken);

	while (tokenizer.NextToken(strToken))
	{
	    if (strToken.Right(1) != ">" && strToken.Right(1) != "." && strToken != "")
	    {
		CTreeNode<CIdString> *pSubTree = new CTreeNode<CIdString>;
		*pTree += pSubTree;

		if (strToken.Right(1) == "/" && nPrune != 0)
		{	    
		    ReadTree(pSubTree, strToken, nPrune);
		}

		pSubTree->Node() = strToken;
		pSubTree->Node().SetId(m_nId++);
	    }
	}
    }
}

void CRecoverView::ReadTree(CTreeNode<CIdString> *pTree, const char *strDir, int nPrune)
{
    CStrings strDirectory = strDir;

    strDirectory = strDirectory.Mid(11, strDirectory.Length()-2);

    Write("cd \""+strDirectory+"\"\n");
    CStrings strRead = Write("ls\n");
    
    WorkListing(pTree, strRead, nPrune);

    Write("cd ..\n");
}

void CRecoverView::Recover()
{
    CWaitCursor *cursor;

    cursor = new CWaitCursor(Parent());

    Write("clear\n");

    CSimpleObjectList<CIdString> list;

    m_pFilesBox->GetCheckedItems(list);

    if (list.card() > 0 && m_pFilesBox->Tree() != NULL)
    {
	for (int nIndex = 0; nIndex < list.card(); nIndex++)
	{
	    CIdString *pString = list[nIndex];

	    CGraphicalTreeNode *pNode = m_pFilesBox->Tree()->GetById(pString->Id());

	    if (pNode != NULL)
	    {
		CStrings strPath = FilePath(pNode);
		CStrings strFile = File(pNode);

	        Write("cd "+strPath+"\n");
		Write("add "+strFile+"\n");
	    }
	}
    }

    CStrings strRead = Write("list\n");

    CStringTokenizer tokenizer("\n");
    tokenizer << strRead;
    CStrings strToken;
    tokenizer.NextToken(strToken);

    CStrings strResult;

    while (tokenizer.NextToken(strToken))
    {
	if (strToken.Right(1) != ">" && strToken != "")
	{
	    strResult += strToken+"\n";
	}
    }

    delete cursor;

    CRecoverAskDialog dialog(strResult);
    dialog.Create();

    int nResult = dialog.DoModal();

    dialog.Unmanage();

    if (nResult == CDialog::IDOK)
    {
	cursor = new CWaitCursor(Parent());

	Write("extract\n");
	CStrings strRead = Write("y\n");

	while (strRead.Right(1) != ">")
	{
	    CMessageDialog dialog(CMessageDialog::QUESTION);
	    dialog.Create();

	    int nNewLine = -1;
	    CStrings strMessage;
	    int nIndex;

	    if (strRead.Right(4) != "[yn]")
	    {
		for (nIndex = strRead.Length()-1; nIndex > 0; nIndex--)
		{
		    if (strRead[nIndex] == '\n')
		    {
			if (nNewLine >= 0)
			{
			    break;
			}
			else
			{
			    while (nIndex > 0 && strRead[nIndex] == '\n')
			    {
				nIndex--;
			    }
			    nNewLine = nIndex;			
			}
		    }
		}
		strMessage = strRead.Mid(nIndex+1, nNewLine);

		dialog.SetText(strMessage);
	
		delete cursor;

		int nResult = dialog.DoModal();

		dialog.Unmanage();

		cursor = new CWaitCursor(Parent());

		if (nResult != CDialog::IDYES)
		{
		    Write("n\n", false);
		    break;
		}
		else
		{
		    strRead = Write("y\n");
		}
	    }
	    else
	    {
  		strRead = Write("n\n");

//  		strRead = Read();
	    }
	}

	delete cursor;
    }
}

CStrings CRecoverView::File(CGraphicalTreeNode *pNode) return strFile
{
    strFile = pNode->Node();

    if (strFile.Right(1) == "/")
    {
	strFile = strFile.Left(strFile.Length()-1);
    }

    strFile = strFile.Right(strFile.Length() - 11);
}

CStrings CRecoverView::FilePath(CGraphicalTreeNode *pNode, bool bDisk)
{
    CStrings strPath;

    if (bDisk)
    {
	strPath = m_strDisk;
	if (strPath.Right(1) != "/")
	{
	    strPath += "/";
	}
    }

    CStrings strNewPath = m_strPath;
    if (strNewPath[0] == '/')
    {
	strNewPath = strNewPath.Mid(1);
    }

//     strPath += m_strPath;
    strPath += strNewPath;
    if (strPath.Right(1) != "/")
    {
	strPath += "/";
    }
    
    CStrings strFile;

    CGraphicalTreeNode *pParent = pNode->Parent();

    while (pParent != NULL)
    {
	CStrings strNode = pParent->Node();

	if (strNode != "")
	{
	    if (strNode.Right(1) == "/")
	    {
		strNode = strNode.Left(strNode.Length()-1);
	    }

	    strNode = strNode.Right(strNode.Length()-11);

	    if (strFile == "")
	    {
		strFile = strNode;
	    }
	    else
	    {
		strFile = strNode + "/" + strFile;
	    }

	}
	pParent = pParent->Parent();
    }

    strPath = strPath + strFile;

    return strPath;
}


