/*******************************************************************************
 *
 * amrecGUI Version 0.2.2
 *
 * amrecgui is a simple frontend for amrecover, the command-line recovery tool for the backup suite amanda. amrecgui is licensed under GPL conditions.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#ifndef RECOVERVIEW_H

#define RECOVERVIEW_H

#include <gui/view.h>
#include <stdio.h>
#include <basic/ctree.h>

class CDataSetConfiguration;
class CRecoverTreeBox;
class CButton;
class CLabelledDateTextField;
class CSelectionList;
class CShellInterface;
class CFileButton;
class CLabelledTextField;
class CGraphicalTreeNode;

class CRecoverView: public CView
{
 private:
    CDataSetConfiguration *m_pConfiguration;
    CRecoverTreeBox *m_pFilesBox;
    CButton *m_pDatasetButton;
    CButton *m_pProperties;
    CLabelledDateTextField *m_pDate;
    CButton *m_pCommit;
    CSelectionList *m_pSelectionList;
    CShellInterface *m_pInterface;
    CFileButton *m_pOutput;
    CButton *m_pRecover;
    bool m_bConnected;
    CButton *m_pConnect;
    CButton *m_pDisconnect;
    int m_fds[2];
    int m_outfds[2];
    FILE *m_Command;
    FILE *m_Output;
    CLabelledTextField *m_pPath;
    CStrings m_strPath;
    IDENT m_nId;
    CStrings m_strDisk;
    CSimpleList<CGraphicalTreeNode *> m_LoadedNodes;
 public:
    CRecoverView(CDataSetConfiguration *pConfiguration);
    void Init();
    inline bool Connected() const { return m_bConnected; }
    inline void SetConnected(bool bConnected=true) { m_bConnected = bConnected; }
 protected:
    virtual void InternalCreate(unsigned long lStyle=0L);
    virtual void OnMessage(CMessageClient *pClient, unsigned long lMask, void *pData=NULL);	
    virtual void InternalDestroy();
 private:
    void Connect();
    void Disconnect();
    void Commit();
    CStrings Read();
    CStrings Write(const char *strCommand, bool bRead=true);
    void ReadTree();
    void ReadTree(CTreeNode<CIdString> *pTree, const char *strDir, int nPrune);
    void WorkListing(CTreeNode<CIdString> *pTree, const CStrings &strDirectory, int nPrune);
    void Recover();
    CStrings FilePath(CGraphicalTreeNode *pTree, bool bDisk=true);
    CStrings File(CGraphicalTreeNode *pTree);
    void UpdateListing(CGraphicalTreeNode *pNode);
    void UpdateChildListing(CGraphicalTreeNode *pNode, bool bReuseDir);
};

#endif
